@@ -56,8 +56,6 @@ bool StreamLog::getNextLine(std::string& line) {
     //try and fix the stream
     if(isFinished()) stream->clear();
 
-    char buff[1024];
-
 #ifdef _WIN32
 
     DWORD available_bytes;
@@ -69,9 +67,7 @@ bool StreamLog::getNextLine(std::string& line) {
 
 #endif
 
-    stream->getline(buff, 1024);
-    line = std::string(buff);
-
+    std::getline(*stream, line);
 
     //remove carriage returns
     if (line.size() > 0 && line[line.size()-1] == '\r') {
@@ -79,11 +75,6 @@ bool StreamLog::getNextLine(std::string& line) {
     }
 
     if(isFinished()) {
-        //clear the failbit if only failed because the line was too long
-        if(!stream->bad() && stream->gcount() >= (1024-1)) {
-            stream->clear();
-        }
-
         return false;
     }
 
@@ -156,11 +147,11 @@ float SeekLog::getPercent() {
     return current_percent;
 }
 
-void SeekLog::setPointer(long pointer) {
+void SeekLog::setPointer(std::streampos pointer) {
     stream->seekg(pointer);
 }
 
-long SeekLog::getPointer() {
+std::streampos SeekLog::getPointer() {
     return stream->tellg();
 }
 
@@ -168,12 +159,12 @@ void SeekLog::seekTo(float percent) {
 
     if(isFinished()) stream->clear();
 
-    long mem_pointer = (long) (percent * file_size);
-
-    setPointer(mem_pointer);
+    std::streampos mem_offset = (std::streampos) (percent * file_size);
 
+    setPointer(mem_offset);
+    
     //throw away end of line
-    if(mem_pointer!=0) {
+    if(mem_offset != (std::streampos)0) {
         std::string eol;
         getNextLine(eol);
     }
@@ -184,22 +175,14 @@ bool SeekLog::getNextLine(std::string& line) {
     //try and fix the stream
     if(isFinished()) stream->clear();
 
-    char buff[1024];
-
-    stream->getline(buff, 1024);
-    line = std::string(buff);
-
+    std::getline(*stream, line);
+    
     //remove carriage returns
     if (line.size() > 0 && line[line.size()-1] == '\r') {
         line.resize(line.size() - 1);
     }
 
     if(stream->fail()) {
-        //clear the failbit if only failed because the line was too long
-        if(!stream->bad() && stream->gcount() >= (1024-1)) {
-            stream->clear();
-        }
-
         return false;
     }
 
@@ -211,16 +194,19 @@ bool SeekLog::getNextLine(std::string& line) {
 
 // temporarily move the file pointer to get a line somewhere else in the file
 bool SeekLog::getNextLineAt(std::string& line, float percent) {
-
-    long currpointer = getPointer();
+    stream->clear();
+    
+    std::streampos currpointer = getPointer();
 
     seekTo(percent);
 
     bool success = getNextLine(line);
 
+    stream->clear();
+
     //set the pointer back
     setPointer(currpointer);
-
+    
     return success;
 }
 
