@@ -26,7 +26,7 @@ void GourceSettings::help(bool extended_help) {
     SDLAppCreateWindowsConsole();
 
     //resize window to fit help message
-    SDLAppResizeWindowsConsole(800);
+    SDLAppResizeWindowsConsole(820);
 #endif
 
     printf("Gource v%s\n", GOURCE_VERSION);
@@ -36,7 +36,8 @@ void GourceSettings::help(bool extended_help) {
     printf("  -h, --help                       Help\n\n");
     printf("  -WIDTHxHEIGHT, --viewport        Set viewport size\n");
     printf("  -f, --fullscreen                 Fullscreen\n");
-    printf("      --multi-sampling             Enable multi-sampling\n\n");
+    printf("      --multi-sampling             Enable multi-sampling\n");
+    printf("      --no-vsync                   Disable vsync\n\n");
 
     printf("  -p, --start-position POSITION    Begin at some position (0.0-1.0 or 'random')\n");
     printf("      --stop-position  POSITION    Stop at some position\n");
@@ -100,7 +101,7 @@ if(extended_help) {
     printf("  --git-branch             Get the git log of a particular branch\n\n");
 
     printf("  --hide DISPLAY_ELEMENT   bloom,date,dirnames,files,filenames,mouse,progress,\n");
-    printf("                           tree,users,usernames\n\n");
+    printf("                           root,tree,users,usernames\n\n");
 
     printf("  --logo IMAGE             Logo to display in the foreground\n");
     printf("  --logo-offset XxY        Offset position of the logo\n\n");
@@ -122,7 +123,7 @@ if(extended_help) {
     printf("  --highlight-dirs         Highlight the names of all directories\n");
     printf("  --highlight-colour       Font colour for highlighted text\n\n");
 
-    printf("  --hash-seed SEED         Change the seed of hash function.\n\n");
+    printf("  --hash-seed SEED         Change the seed of hash function\n\n");
 
     printf("  --path PATH\n\n");
 }
@@ -163,10 +164,10 @@ GourceSettings::GourceSettings() {
     arg_aliases["H"] = "extended-help";
     arg_aliases["b"] = "background-colour";
     arg_aliases["c"] = "time-scale";
-    arg_aliases["background"] = "background-colour";
-    arg_aliases["disable-bloom"]    = "hide-bloom";
-    arg_aliases["disable-progress"] = "hide-progress";
-    arg_aliases["highlight-users"] = "highlight-all-users";
+    arg_aliases["background"]          = "background-colour";
+    arg_aliases["disable-bloom"]       = "hide-bloom";
+    arg_aliases["disable-progress"]    = "hide-progress";
+    arg_aliases["highlight-all-users"] = "highlight-users";
 
     //command line only options
     conf_sections["help"]            = "command-line";
@@ -201,10 +202,12 @@ GourceSettings::GourceSettings() {
     arg_types["hide-progress"]   = "bool";
     arg_types["hide-bloom"]      = "bool";
     arg_types["hide-mouse"]      = "bool";
-    arg_types["highlight-all-users"] = "bool";
+    arg_types["hide-root"]       = "bool";
+    arg_types["highlight-users"] = "bool";
     arg_types["highlight-dirs"]  = "bool";
     arg_types["file-extensions"] = "bool";
     arg_types["key"]             = "bool";
+    arg_types["ffp"]             = "bool";
 
     arg_types["disable-auto-rotate"] = "bool";
     arg_types["disable-auto-skip"]   = "bool";
@@ -268,6 +271,8 @@ void GourceSettings::setGourceDefaults() {
 
     path = ".";
 
+    ffp = false;
+
     hide_date      = false;
     hide_users     = false;
     hide_tree      = false;
@@ -275,9 +280,10 @@ void GourceSettings::setGourceDefaults() {
     hide_usernames = false;
     hide_filenames = false;
     hide_dirnames  = false;
-    hide_progress = false;
-    hide_bloom    = false;
-    hide_mouse    = false;
+    hide_progress  = false;
+    hide_bloom     = false;
+    hide_mouse     = false;
+    hide_root      = false;
 
     start_position = 0.0f;
     stop_position  = 0.0f;
@@ -330,7 +336,7 @@ void GourceSettings::setGourceDefaults() {
     log_format  = "";
     date_format = "%A, %d %B, %Y %X";
 
-    max_files      = 1000;
+    max_files      = 0;
     max_user_speed = 500.0f;
     max_file_lag   = 5.0f;
 
@@ -481,7 +487,8 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
                && hide_field != "dirnames"
                && hide_field != "bloom"
                && hide_field != "progress"
-               && hide_field != "mouse") {
+               && hide_field != "mouse"
+               && hide_field != "root") {
                 std::string unknown_hide_option = std::string("unknown option hide ") + hide_field;
                 conffile.entryException(entry, unknown_hide_option);
             }
@@ -513,6 +520,7 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
             else if(hidestr == "dirnames")  hide_dirnames  = true;
             else if(hidestr == "bloom")     hide_bloom     = true;
             else if(hidestr == "progress")  hide_progress  = true;
+            else if(hidestr == "root")      hide_root      = true;
             else if(hidestr == "mouse")     {
                 hide_mouse     = true;
                 hide_progress  = true;
@@ -897,6 +905,10 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         show_key = true;
     }
 
+    if(gource_settings->getBool("ffp")) {
+        ffp = true;
+    }
+
     if(gource_settings->getBool("realtime")) {
         days_per_second = 1.0 / 86400.0;
     }
@@ -971,7 +983,8 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
-    if(gource_settings->getBool("highlight-all-users")) {
+    if(   gource_settings->getBool("highlight-users")
+       || gource_settings->getBool("highlight-all-users")) {
         highlight_all_users = true;
     }
 
