@@ -18,7 +18,7 @@
 #ifndef GOURCE_SETTINGS_H
 #define GOURCE_SETTINGS_H
 
-#define GOURCE_VERSION "0.32"
+#define GOURCE_VERSION "0.34"
 
 #include <dirent.h>
 
@@ -48,11 +48,12 @@ public:
     bool hide_progress;
     bool hide_bloom;
     bool hide_mouse;
+    bool hide_root;
 
     bool disable_auto_rotate;
 
     bool show_key;
-    
+
     std::string load_config;
     std::string save_config;
     std::string path;
@@ -74,6 +75,8 @@ public:
 
     bool loop;
 
+    bool ffp;
+
     bool colour_user_images;
     std::string default_user_image;
     std::string user_image_dir;
@@ -115,13 +118,13 @@ public:
     bool highlight_dirs;
     bool highlight_all_users;
     vec3f highlight_colour;
-   
+
     std::vector<std::string> highlight_users;
     std::vector<std::string> follow_users;
     std::vector<Regex*> file_filters;
     std::vector<Regex*> user_filters;
     bool file_extensions;
-    
+
     std::string output_custom_filename;
 
     TextureResource* file_graphic;
